#!/usr/bin/env python

import sys
import pymysql.cursors
import re
from dmr_utils.logger import get_logger
from ocs_db.models import *

logger = get_logger()


class DatabaseConnect:

    def __init__(self, host, user, password, database):

        self.logger = logger
        try:
            self.connection = pymysql.connect(host=host, user=user, password=password, db=database,\
                                              cursorclass=pymysql.cursors.DictCursor)
        except Exception as e:
            self.logger.send_log("error", "An error occured while connecting to MySQL Server : {} : \
            line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))

    def query_db(self, query):

        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query)
                return cursor.fetchall()
        except Exception as e:
            self.logger.send_log("error", "An error occured while executing query on MySQL Server : {} \
            : line : {}".format(str(e), str(sys.exc_info()[2].tb_lineno)))

    def close_db(self):

        self.connection.close()


class OcsNgOps:

    def __init__(self, db_host, db_user, db_pass, db):

        self.db = DatabaseConnect(db_host, db_user, db_pass, db)

    def check_values(self, value_list, value_to_compare):

        for value in value_list:
            if re.search(value.lower(), value_to_compare.lower()) is None:
                continue
            else:
                return True
        return False

    def read_values(self, file_path):

        values = []
        with open(file_path, "r") as value_file:
            lines = value_file.readlines()
        for line in lines:
            values.append(line.strip())
        return values

    def get_softwares(self):

        softwares = self.db.query_db("SELECT id,hardware_id,name,version,installdate FROM softwares")
        return softwares

    def get_softwares_by_host(self, host_name):

        softwares = self.db.query_db("SELECT softwares.id,hardware_id,softwares.name,version,installdate FROM softwares\
                                    ,hardware WHERE softwares.hardware_id = hardware.id and \
                                     hardware.name = \"" + host_name + "\"")

        return softwares

    def get_software(self, sw_id):

        software = self.db.query_db("SELECT id,hardware_id,name,version,installdate FROM softwares WHERE id="+sw_id)
        return software

    def get_hardware(self, hw_id):

        hardware = self.db.query_db("SELECT id,deviceid,name,osname,osversion,ipaddr,processort FROM \
        hardware WHERE id=" + str(hw_id))
        return hardware

    def get_hardwares(self):
        hardwares = self.db.query_db("SELECT name FROM hardware")
        return hardwares

    def get_hardware_by_name(self, host_name):
        hardware = self.db.query_db("SELECT * FROM hardware WHERE name=\"" + host_name + "\"")
        return hardware

    def get_device_hw_info(self, sql_table, hw_id):
        hw_info = self.db.query_db("SELECT * FROM " + sql_table + " WHERE hardware_id=" + str(hw_id))
        return hw_info

    def get_device_by_id(self, sql_table, record_id):
        hw_info = self.db.query_db("SELECT * FROM " + sql_table + " WHERE id=" + str(record_id))
        return hw_info

    def software_as_object(self, software_dict, hardware):
        software = Software()
        software.s_id = software_dict["id"]
        software.install_date = software_dict["installdate"]
        software.hardware_id = hardware
        software.name = software_dict["name"]
        software.version = software_dict["version"]

        return software

    def hardware_as_object(self, hardware_dict):
        hardware = Hardware()
        hardware.h_id = hardware_dict["id"]
        hardware.device_id = hardware_dict["deviceid"]
        hardware.osname = hardware_dict["osname"]
        hardware.osversion = hardware_dict["osversion"]
        hardware.processort = hardware_dict["processort"]
        hardware.ipaddr = hardware_dict["ipaddr"]
        hardware.name = hardware_dict["name"]

        return hardware

    def check_hw_info_whitelist(self, db_record, db_table, db_fields):
        hw_whitelist = HardwareWhiteList.objects.filter(table_name=db_table)
        if len(hw_whitelist) == 0:
            return False
        whitelist = True
        for field in db_fields:
            if len(hw_whitelist.filter(context__icontains=db_record[field])) == 0:
                whitelist = False
                break
        return whitelist

    def check_hw_info_blacklist(self, db_record, db_table, db_fields):
        hw_blacklist = HardwareBlackList.objects.filter(table_name=db_table)
        if len(hw_blacklist) == 0:
            return False
        blacklist = True
        for field in db_fields:
            if len(hw_blacklist.filter(context__icontains=db_record[field])) == 0:
                blacklist = False
                break
        return blacklist

    def send_log(self, software, hardware):

        self.db.logger.send_log("error", "Found a vulnerable software : {} on {}({}) \
        Machine".format(software['name'], hardware['name'], hardware['deviceid']))


def create_as_context(sw, hw):
    blacklist = False
    whitelist = False
    if SoftwareBlackList.objects.get(software=sw['id']):
        blacklist = True
    elif SoftwareWhiteList.objects.get(software=sw['id']):
        whitelist = True
    context_dict = {"software": sw, "hardware": hw, "blacklist": blacklist, "whitelist": whitelist}
    return context_dict