# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.auth.models import User
from datetime import datetime


class Hardware(models.Model):

    h_id = models.IntegerField(verbose_name="ocs ID")
    device_id = models.CharField(max_length=255, verbose_name="Cihaz ID")
    name = models.CharField(max_length=255, verbose_name=u"Cihaz İsmi")
    osname = models.CharField(max_length=100, verbose_name=u"İşletim Sistemi")
    osversion = models.CharField(max_length=60, verbose_name=u"İşletim Sistemi Sürümü")
    ipaddr = models.CharField(max_length=20, verbose_name="IP Adresi")
    processort = models.CharField(max_length=255, verbose_name=u"İşlemci")

    class Meta:
        verbose_name = u"Donanım"
        verbose_name_plural = u"Donanımlar"

    def __str__(self):
        return self.name + " |**| " + self.ipaddr


class Software(models.Model):

    s_id = models.IntegerField(verbose_name="ocs ID")
    hardware_id = models.ForeignKey(Hardware, on_delete=models.DO_NOTHING, verbose_name=u"Bulunduğu Donanım")
    name = models.CharField(max_length=60, verbose_name=u"Yazılım")
    version = models.CharField(max_length=60, verbose_name=u"Sürüm")
    install_date = models.DateTimeField(verbose_name=u"Yüklendiği Tarih", null=True)

    class Meta:
        verbose_name = u"Yazılım"
        verbose_name_plural = u"Yazılımlar"

    def __str__(self):
        return self.name

# Blacklist for Softwares
class SoftwareBlackList(models.Model):

    software = models.ForeignKey(Software, on_delete=models.CASCADE, verbose_name=u"Yazılım")
    hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, verbose_name=u"Donanım")
    added_by = models.ForeignKey(User, on_delete=models.DO_NOTHING, verbose_name=u"Ekleyen")
    date = models.DateTimeField(default=datetime.now(), verbose_name=u"Tarih")

    class Meta:
        verbose_name = "Yazılım Kara Liste"
        verbose_name_plural = "Yazılım Kara Listeler"

    def __str__(self):
        return self.software.name + "|**|" + self.hardware.name

# Whitelist For Softwares
class SoftwareWhiteList(models.Model):

    software = models.ForeignKey(Software, on_delete=models.CASCADE, verbose_name=u"Yazılım")
    hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, verbose_name=u"Donanım")
    added_by = models.ForeignKey(User, on_delete=models.DO_NOTHING, verbose_name=u"Ekleyen")
    date = models.DateTimeField(default=datetime.now(), verbose_name=u"Tarih")

    class Meta:
        verbose_name = "Yazılım Beyaz Liste"
        verbose_name_plural = "Yazılım Beyaz Listeler"

    def __str__(self):
        return self.software.name + "|**|" + self.hardware.name


class ContextToShow(models.Model):
    software = models.ForeignKey(Software, on_delete=models.DO_NOTHING, verbose_name=u"Yazılım")
    hardware = models.ForeignKey(Hardware, on_delete=models.DO_NOTHING, verbose_name=u"Donanım")
    whitelist = models.BooleanField(default=False)
    blacklist = models.BooleanField(default=False)

    class Meta:
        abstract = True


class HardwareInfo(models.Model):

    hardware = models.ForeignKey(Hardware, on_delete=models.DO_NOTHING, verbose_name=u"Donanım")

    bios = []
    controllers = []
    devices = []
    drives = []
    inputs = []
    memories = []
    modems = []
    monitors = []
    networks = []
    ports = []
    printers = []
    slots = []
    sounds = []
    storages = []
    videos = []

    class Meta:
        abstract = True


class HardwareBlackList(models.Model):
    table_name = models.CharField(max_length=55, verbose_name=u"Tablo Adı")
    context = models.TextField(verbose_name=u"Filtrelenmiş İçerik")
    added_by = models.ForeignKey(User, on_delete=models.DO_NOTHING, verbose_name=u"Ekleyen")
    date = models.DateTimeField(default=datetime.now(), verbose_name=u"Tarih")

    class Meta:
        verbose_name = "Donanım Kara Liste"
        verbose_name_plural = "Donanım Kara Listeler"

    def __str__(self):
        return self.context


class HardwareWhiteList(models.Model):
    table_name = models.CharField(max_length=55, verbose_name=u"Tablo Adı")
    context = models.TextField(verbose_name=u"Filtrelenmiş İçerik")
    added_by = models.ForeignKey(User, on_delete=models.DO_NOTHING, verbose_name=u"Ekleyen")
    date = models.DateTimeField(default=datetime.now(), verbose_name=u"Tarih")

    class Meta:
        verbose_name = "Donanım Beyaz Liste"
        verbose_name_plural = "Donanım Beyaz Listeler"

    def __str__(self):
        return self.context
