# -*- coding: utf-8 -*-
from django.shortcuts import render, HttpResponseRedirect, HttpResponse
from django.core.paginator import PageNotAnInteger, Paginator, EmptyPage
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.decorators import login_required

# Create your views here.

import dmr_utils.ocsng as ocs_ng
from ocs_db.models import *
import re
import sys
from dmr_utils import config_parser


@login_required
def show_softwares(request):

    try:
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        ocs = ocs_ng.OcsNgOps(db_host, db_user, db_pass, db_name)
        hardwares = ocs.get_hardwares()
        logger = ocs_ng.get_logger()
        host_name = request.GET.get('host')
        filter_sw = request.GET.get('filter_software')
        if host_name is None or host_name == "None" or host_name == "":
            softwares = ocs.get_softwares()
        else:
            softwares = ocs.get_softwares_by_host(host_name)
        context_list = []
        if softwares is not None:
            for software in softwares:
                hardware = ocs.get_hardware(software["hardware_id"])
                hardware = hardware[0]
                context = ContextToShow()
                context.software = Software()
                context.hardware = Hardware()
                context.hardware.h_id = hardware["id"]
                context.hardware.device_id = hardware["deviceid"]
                context.hardware.osname = hardware["osname"]
                context.hardware.osversion = hardware["osversion"]
                context.hardware.processort = hardware["processort"]
                context.hardware.ipaddr = hardware["ipaddr"]
                context.hardware.name = hardware["name"]
                context.software.s_id = software["id"]
                context.software.install_date = software["installdate"]
                context.software.hardware_id = context.hardware
                context.software.name = software["name"]
                context.software.version = software["version"]
                try:
                    sw = Software.objects.get(name=software["name"], version=context.software.version)
                    wl_obj = SoftwareWhiteList.objects.get(software__name=sw.name)
                    context.whitelist = True
                except ObjectDoesNotExist:
                    pass
                except Exception as e:
                    error_message = "ERROR while getting whitelist details ::: {0} ::: line {1}"\
                                        .format(str(e), sys.exc_info()[2].tb_lineno)
                    logger.send_log("error", error_message)
                    return render(request, "show_env.html", {"message": error_message})
                try:
                    sw = Software.objects.get(name=software["name"], version=context.software.version)
                    bl_obj = SoftwareBlackList.objects.get(software__name=sw.name)
                    context.blacklist = True
                except ObjectDoesNotExist:
                    pass
                except Exception as e:
                    error_message = "ERROR while getting blacklist details ::: {0} ::: line {1}"\
                                        .format(str(e), sys.exc_info()[2].tb_lineno)
                    logger.send_log("error", error_message)
                    return render(request, "show_env.html", {"message": error_message})
                context_list.append(context)

        context_list_filtered = []
        if filter_sw == "all":
            context_list_filtered = context_list
        elif filter_sw == "black":
            for c in context_list:
                if c.blacklist:
                    context_list_filtered.append(c)
        elif filter_sw == "white":
            for c in context_list:
                if c.whitelist:
                    context_list_filtered.append(c)
        else:
            context_list_filtered = context_list
        paginator = Paginator(context_list_filtered, 12)

        page = request.GET.get('page')
        try:
            context = paginator.page(page)
        except PageNotAnInteger:
            context = paginator.page(1)
        except EmptyPage:
            context = paginator.page(paginator.num_pages)
        except Exception as e:
            error_message = "ERROR while paging context ::: {0} ::: line {1}"\
                                .format(str(e), sys.exc_info()[2].tb_lineno)
            logger.send_log("error", error_message)
            return render(request, "show_env.html", {"message": error_message})

        return render(request, "show_env.html", {"context": context, "host": host_name, "filter": filter_sw, "hardwares": hardwares})
    except Exception as e:
        error_message = "ERROR while getting and listing softwares from mysql ::: {0} ::: line {1}"\
                            .format(str(e), sys.exc_info()[2].tb_lineno)
        logger.send_log("error", error_message)
        return render(request, "show_env.html", {"message": error_message})


@login_required
def operate_softwares(request):

    try:
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        ocs = ocs_ng.OcsNgOps(db_host, db_user, db_pass, db_name)
        logger = ocs_ng.get_logger()
        post_values = request.POST

        previous_page = request.GET.get('page')

        if previous_page is None:
            redirect_to = "/ocs/softwares/"
        else:
            redirect_to = unicode(previous_page)

        for key, value in post_values.iteritems():
            if key != "csrfmiddlewaretoken":
                if re.search("list_", key) is not None:
                    sw_name = key.split("_")[1]
                    sw_version = key.split("_")[2]
                    software_id = key.split("_")[3]
                    try:
                        software = Software.objects.get(name=sw_name, version=sw_version)
                    except ObjectDoesNotExist:
                        software = None
                    except Exception as e:
                        error_message = "ERROR while getting software details ::: {0} ::: line {1}"\
                                            .format(str(e), sys.exc_info()[2].tb_lineno)
                        logger.send_log("error", error_message)
                        return render(request, "show_env.html", {"message": error_message})

                    if software is None:
                        sw_dict = ocs.get_software(str(software_id))[0]
                        sw_hardware = ocs.get_hardware(sw_dict["hardware_id"])[0]
                        hardware = ocs.hardware_as_object(sw_hardware)
                        try:
                            hardware = Hardware.objects.get(name=hardware.name)
                        except ObjectDoesNotExist:
                            hardware = ocs.hardware_as_object(sw_hardware)
                            hardware.save()
                        except Exception as e:
                            error_message = "ERROR while saving hardware details ::: {0} ::: line {1}"\
                                                .format(str(e), sys.exc_info()[2].tb_lineno)
                            logger.send_log("error", error_message)
                            return render(request, "show_env.html", {"message": error_message})
                        software = ocs.software_as_object(sw_dict, hardware)
                        software.save()

                    if value == "white":
                        try:
                            bl_obj = SoftwareBlackList.objects.get(software__name=software.name)
                            bl_obj.delete()
                        except ObjectDoesNotExist:
                            pass
                        except Exception as e:
                            error_message = "ERROR while querying blacklist object ::: {0} ::: line {1}"\
                                                .format(str(e), sys.exc_info()[2].tb_lineno)
                            logger.send_log("error", error_message)
                            return render(request, "show_env.html", {"message": error_message})
                        finally:
                            try:
                                SoftwareWhiteList.objects.get(software__name=software.name)
                            except ObjectDoesNotExist:
                                wl_obj = SoftwareWhiteList()
                                wl_obj.software = software
                                wl_obj.hardware = software.hardware_id
                                wl_obj.added_by = request.user
                                wl_obj.save()
                                log_message = wl_obj.software.name + u" yazılımı " + request.user.username + \
                                              u" tarafından Beyaz Liste'ye eklendi"
                                logger.send_log("info", log_message)
                            except Exception as e:
                                error_message = "ERROR while adding to whitelist ::: {0} ::: line {1}"\
                                                    .format(str(e), sys.exc_info()[2].tb_lineno)
                                logger.send_log("error", error_message)
                                return render(request, "show_env.html", {"message": error_message})
                    elif value == "black":
                        try:
                            wl_obj = SoftwareWhiteList.objects.get(software__name=software.name)
                            wl_obj.delete()
                        except ObjectDoesNotExist:
                            pass
                        except Exception as e:
                            error_message = "ERROR while querying whitelist object ::: {0} ::: line {1}"\
                                                .format(str(e), sys.exc_info()[2].tb_lineno)
                            logger.send_log("error", error_message)
                            return render(request, "show_env.html", {"message": error_message})
                        finally:
                            try:
                                SoftwareBlackList.objects.get(software__name=software.name)
                            except ObjectDoesNotExist:
                                bl_obj = SoftwareBlackList()
                                bl_obj.software = software
                                bl_obj.hardware = software.hardware_id
                                bl_obj.added_by = request.user
                                bl_obj.save()
                                log_message = bl_obj.software.name + u" yazılımı " + request.user.username + \
                                              u" tarafından Kara Liste'ye eklendi"
                                logger.send_log("info", log_message)
                            except Exception as e:
                                error_message = "ERROR while adding to blacklist ::: {0} ::: line {1}"\
                                                    .format(str(e), sys.exc_info()[2].tb_lineno)
                                logger.send_log("error", error_message)
                                return render(request, "show_env.html", {"message": error_message})

        return HttpResponseRedirect(redirect_to)
    except Exception as e:
        error_message = "ERROR while operating software request ::: {0} ::: line {1}"\
                            .format(str(e), sys.exc_info()[2].tb_lineno)
        logger.send_log("error", error_message)
        return render(request, "show_env.html", {"message": error_message})


@login_required
def hardware_info(request):
    try:
        BIOS_FIELDS = ['SMANUFACTURER', 'BMANUFACTURER', 'BVERSION', 'SSN', 'SMODEL']
        CONTROLLERS_FIELDS = ['NAME', 'MANUFACTURER', 'CAPTION', 'TYPE']
        DEVICES_FIELDS = ['NAME']
        DRIVES_FIELDS = ['FILESYSTEM', 'VOLUMN', 'TYPE']
        INPUTS_FIELDS = ['CAPTION', 'DESCRIPTION']
        MEMORIES_FIELDS = ['CAPTION', 'DESCRIPTION']
        MODEMS_FIELDS = ['NAME', 'DESCRIPTION', 'MODEL']
        MONITORS_FIELDS = ['MANUFACTURER', 'CAPTION', 'DESCRIPTION', 'TYPE']
        NETWORKS_FIELDS = ['MACADDR', 'DESCRIPTION']
        PORTS_FIELDS = ['CAPTION', 'DESCRIPTION', 'TYPE']
        PRINTERS_FIELDS = ['NAME', 'DRIVER', 'PORT']
        SLOTS_FIELDS = ['NAME', 'DESCRIPTION']
        SOUNDS_FIELDS = ['MANUFACTURER', 'NAME', 'DESCRIPTION']
        STORAGES_FIELDS= ['MANUFACTURER', 'NAME', 'MODEL', 'DESCRIPTION']
        VIDEOS_FIELDS = ['NAME', 'CHIPSET']

        host_name = request.GET.get('host')
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        ocs = ocs_ng.OcsNgOps(db_host, db_user, db_pass, db_name)
        if host_name is None or host_name == "None" or host_name == "":
            hw_info = []
            hardwares = ocs.get_hardwares()
        else:
            logger = ocs_ng.get_logger()
            hw_info = HardwareInfo()
            hw_id = ocs.get_hardware_by_name(host_name)
            if len(hw_id) == 0:
                hw_info = []
                hardwares = ocs.get_hardwares()
            else:
                hw_id = hw_id[0]['ID']
                hw = ocs.get_hardware(hw_id)
                hw_info.hardware = ocs.hardware_as_object(hw[0])
                # check hw_id
                hw_info.bios = ocs.get_device_hw_info('bios', hw_id)
                for b in hw_info.bios:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'bios', BIOS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'bios', BIOS_FIELDS)
                hw_info.controllers = ocs.get_device_hw_info('controllers', hw_id)
                for b in hw_info.controllers:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'controllers', CONTROLLERS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'controllers', CONTROLLERS_FIELDS)
                hw_info.devices = ocs.get_device_hw_info('devices', hw_id)
                for b in hw_info.devices:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'devices', DEVICES_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'devices', DEVICES_FIELDS)
                hw_info.drives = ocs.get_device_hw_info('drives', hw_id)
                for b in hw_info.drives:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'drives', DRIVES_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'drives', DRIVES_FIELDS)
                hw_info.inputs = ocs.get_device_hw_info('inputs', hw_id)
                for b in hw_info.inputs:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'inputs', INPUTS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'inputs', INPUTS_FIELDS)
                hw_info.memories = ocs.get_device_hw_info('memories', hw_id)
                for b in hw_info.memories:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'memories', MEMORIES_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'memories', MEMORIES_FIELDS)
                hw_info.modems = ocs.get_device_hw_info('modems', hw_id)
                for b in hw_info.modems:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'modems', MODEMS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'modems', MODEMS_FIELDS)
                hw_info.monitors = ocs.get_device_hw_info('monitors', hw_id)
                for b in hw_info.monitors:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'monitors', MONITORS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'monitors', MONITORS_FIELDS)
                hw_info.networks = ocs.get_device_hw_info('networks', hw_id)
                for b in hw_info.networks:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'networks', NETWORKS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'networks', NETWORKS_FIELDS)
                hw_info.ports = ocs.get_device_hw_info('ports', hw_id)
                for b in hw_info.ports:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'ports', PORTS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'ports', PORTS_FIELDS)
                hw_info.printers = ocs.get_device_hw_info('printers', hw_id)
                for b in hw_info.printers:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'printers', PRINTERS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'printers', PRINTERS_FIELDS)
                hw_info.slots = ocs.get_device_hw_info('slots', hw_id)
                for b in hw_info.slots:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'slots', SLOTS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'slots', SLOTS_FIELDS)
                hw_info.sounds = ocs.get_device_hw_info('sounds', hw_id)
                for b in hw_info.sounds:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'sounds', SOUNDS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'sounds', SOUNDS_FIELDS)
                hw_info.storages = ocs.get_device_hw_info('storages', hw_id)
                for b in hw_info.storages:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'storages', STORAGES_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'storages', STORAGES_FIELDS)
                hw_info.videos = ocs.get_device_hw_info('videos', hw_id)
                for b in hw_info.videos:
                    b['blacklist'] = ocs.check_hw_info_blacklist(b, 'videos', VIDEOS_FIELDS)
                    b['whitelist'] = ocs.check_hw_info_whitelist(b, 'videos', VIDEOS_FIELDS)

                hardwares = ocs.get_hardwares()

        return render(request, "show_hw_env.html", {"context": hw_info, 'hardwares': hardwares})
    except Exception as e:
        error_message = "ERROR while operating hardware information ::: {0} ::: line {1}"\
                            .format(str(e), sys.exc_info()[2].tb_lineno)
        logger.send_log("error", error_message)
        return render(request, "show_hw_env.html", {"message": error_message})


@login_required
def operate_hw(request):

    try:
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        ocs = ocs_ng.OcsNgOps(db_host, db_user, db_pass, db_name)
        logger = ocs_ng.get_logger()
        post_values = request.POST

        previous_page = request.GET.get('page')

        if previous_page is None:
            redirect_to = "/ocs/hardware/"
        else:
            redirect_to = unicode(previous_page)

        for key, value in post_values.iteritems():
            if key != "csrfmiddlewaretoken":
                if re.search("list_", key) is not None:
                    hw_table = key.split("_")[1]
                    hw_record_id = key.split("_")[2]
                    hw_id = key.split("_")[3]
                    if hw_table == "bios":
                        record = ocs.get_device_hw_info(hw_table, hw_id)[0]
                    else:
                        record = ocs.get_device_by_id(hw_table, hw_record_id)[0]
                    record_context = ""
                    for field, f_value in record.iteritems():
                        record_context = record_context + unicode(f_value) + u"\n"
                    if value == "white":
                        try:
                            bl_obj = HardwareBlackList.objects.get(table_name=hw_table, context=record_context)
                            bl_obj.delete()
                        except ObjectDoesNotExist:
                            pass
                        except Exception as e:
                            error_message = "ERROR while querying blacklist object ::: {0} ::: line {1}"\
                                                .format(str(e), sys.exc_info()[2].tb_lineno)
                            logger.send_log("error", error_message)
                            return render(request, "show_hw_env.html", {"message": error_message})
                        finally:
                            try:
                                HardwareWhiteList.objects.get(table_name=hw_table, context=record_context)
                            except ObjectDoesNotExist:
                                wl_obj = HardwareWhiteList()
                                wl_obj.table_name = hw_table
                                wl_obj.context = record_context
                                wl_obj.added_by = request.user
                                wl_obj.save()
                                log_message = wl_obj.table_name + u" " + request.user.username + \
                                              u" tarafından Beyaz Liste'ye eklendi"
                                logger.send_log("info", log_message)
                            except Exception as e:
                                error_message = "ERROR while adding to whitelist ::: {0} ::: line {1}"\
                                                    .format(str(e), sys.exc_info()[2].tb_lineno)
                                logger.send_log("error", error_message)
                                return render(request, "show_hw_env.html", {"message": error_message})
                    elif value == "black":
                        try:
                            wl_obj = HardwareWhiteList.objects.get(table_name=hw_table, context=record_context)
                            wl_obj.delete()
                        except ObjectDoesNotExist:
                            pass
                        except Exception as e:
                            error_message = "ERROR while querying whitelist object ::: {0} ::: line {1}"\
                                                .format(str(e), sys.exc_info()[2].tb_lineno)
                            logger.send_log("error", error_message)
                            return render(request, "show_hw_env.html", {"message": error_message})
                        finally:
                            try:
                                HardwareBlackList.objects.get(table_name=hw_table, context=record_context)
                            except ObjectDoesNotExist:
                                bl_obj = HardwareBlackList()
                                bl_obj.table_name = hw_table
                                bl_obj.context = record_context
                                bl_obj.added_by = request.user
                                bl_obj.save()
                                log_message = bl_obj.table_name + u" " + request.user.username + \
                                              u" tarafından Kara Liste'ye eklendi"
                                logger.send_log("info", log_message)
                            except Exception as e:
                                error_message = "ERROR while adding to blacklist ::: {0} ::: line {1}"\
                                                    .format(str(e), sys.exc_info()[2].tb_lineno)
                                logger.send_log("error", error_message)
                                return render(request, "show_hw_env.html", {"message": error_message})

        return HttpResponseRedirect(redirect_to)
    except Exception as e:
        error_message = "ERROR while operating hardware request ::: {0} ::: line {1}"\
                            .format(str(e), sys.exc_info()[2].tb_lineno)
        logger.send_log("error", error_message)
        return render(request, "show_hw_env.html", {"message": error_message})


@login_required
def show_config(request):
    try:
        logger = ocs_ng.get_logger()
        config = config_parser.DmrConfigParser()
        context = {}
        context["host"] = config.get_db_configs()["host"]
        context["user"] = config.get_db_configs()["user"]
        context["password"] = config.get_db_configs()["password"]
        context["db_name"] = config.get_db_configs()["db_name"]
        return render(request, "config.html", context)
    except Exception as e:
        error_message = "Error while getting config : \n {} \n:: at line {}.".format(str(e), str(
                sys.exc_info()[2].tb_lineno))
        logger.send_log("error", error_message)
        return HttpResponse("<script>\
                            alert('Yapılandırma Okunurken Bir Hata Oluştu.');\
                            window.location.pathname = \"/\";\
                        </script>")


@login_required
def set_config(request):
    if request.POST:
        try:
            logger = ocs_ng.get_logger()
            config = config_parser.DmrConfigParser()
            new_config = {"host": request.POST["host_name"],
                          "user": request.POST["user"],
                          "password": request.POST["password"],
                          "db_name": request.POST["db_name"]}
            config.set_db_configs(new_config)
            return HttpResponse("<script>\
                            alert('Yapılandırma Kaydedilmiştir.');\
                            window.location.pathname = \"/ocs/config/\";\
                        </script>")
        except Exception as e:
            error_message = "Error while setting config : \n {} \n:: at line {}.".format(str(e), str(
                sys.exc_info()[2].tb_lineno))
            logger.send_log("error", error_message)
            return HttpResponse("<script>\
                            alert('Yapılandırma Kaydedilirken Bir Hata Oluştu.');\
                            window.location.pathname = \"/ocs/config/\";\
                        </script>")